package spiral.digitalize;

import spiral.digitalize.filters.*;
import java.awt.*;
import java.awt.image.*;

/**
 * <p>Title: Spiral analyzis</p>
 * <p>Description: Zajiuje funkce pro pevod obrzku na data spirly</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Doleel
 * @version 1.0
 */

public class Engine
    extends Component {
  private Image img;
  private int thres = 201;
  private Spiral spiral = null;

  /**
   * Rozhran pro zobrazovn a vstup informac pro Engine
   */
  public GUI gui;

  /**
   * Vytvo nov Engine s rozhranm gui
   * @param gui rozhran
   */
  public Engine(GUI gui) {
    this.gui = gui;
  }

  /**
   * Nahraje obrzek ze souboru
   * @param fileName cesta k obrzku
   */
  public void loadImage(String fileName) {
    img = getToolkit().getImage(fileName);
    gui.showStatus("Image loaded");
  }

  /**
   * Ulo vypotanou spirlu do souboru
   * @param fileName jmno souboru
   */
  public void saveData(String fileName) {
    if(spiral != null) {
      spiral.save(fileName);
      gui.showStatus("Data saved");
    }
  }

  /**
   * Vrac obrzek se kterm se pracuje
   * @return obrzek
   */
  public Image getImage() {
    return img;
  }

  /**
   * Provede operaci Threshold (prh) na obrzek
   */
  public void threshold() {
    if (img != null) {
      ImageFilter filter = new ThresholdFilter(thres);
      img = createImage(new FilteredImageSource(img.getSource(), filter));
      gui.showStatus("Threshold done");
    }
    else {
      gui.showStatus("No image loaded");
    }
  }

  /**
   * Provede operaci Dilatace (ztlutn) na obrzek
   */
  public void dilate() {
    if (img != null) {
      ImageFilter filter = new DilateFilter();
      img = createImage(new FilteredImageSource(img.getSource(), filter));
      gui.showStatus("Dilate done");
    }
    else {
      gui.showStatus("No image loaded");
    }
  }

  /**
   * Provede operaci Skeleton (ztenen) na obrzek
   */
  public void skeleton() {
    if (img != null) {
      ImageFilter filter = new SkeletonFilter();
      img = createImage(new FilteredImageSource(img.getSource(), filter));
      gui.showStatus("Skeleton done");
    }
    else {
      gui.showStatus("No image loaded");
    }
  }

  /**
   * Provede vechny operace na obrzek (1x threshold, 2x dilatace, 10x skeleton)
   */
  public void allInOne() {
    this.threshold();
    this.dilate();
    this.dilate();
    for(int i = 0; i < 10; i++) {
      this.skeleton();
    }
//    this.findSpiral();
  }

  public void findSpiral() {
    spiral = new Spiral(this, img);
  }
}