package spiral.digitalize;

import java.awt.geom.AffineTransform;
import java.util.*;
import java.io.*;
import java.awt.Image;
import java.awt.image.*;

/**
 * <p>Title: Spiral analyzis</p>
 * <p>Description: Tda se pokus najt spirlu v obrzku</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Doleel
 * @version 1.0
 */

public class Spiral {
  private SpiralVector spiral = null;
  private Engine engine;
  private BufferedImage img;
  private Row begin = new Row(0,0);

  /**
   * Najde pro Engine v obrzku spiralu
   * @param e volajci Engine
   * @param i obrzek, kde hledat
   */
  public Spiral(Engine e, Image i) {
    engine = e;
    engine.gui.showStatus("Zanm potat spirlu");
    this.img = new BufferedImage(i.getWidth(null), i.getHeight(null),
                                        BufferedImage.TYPE_INT_RGB);
    while (!img.createGraphics().drawImage(i, new AffineTransform(), null)) {
      ;
    }
    this.start();
  }

  // zacne hledat spiralu
  private void start() {
    Row r = new Row(1,1), r1, r2;
    do {
      r = getStart(r); // najdi
      if (r.distance2(begin) == 0) { // vylezli jsme z obrazku
        engine.gui.showStatus("Nelze najit spiralu");
        return;
      }
      // nalezen stedu
      spiral = getVect(r, begin, 10); // najdi zacatek/konec spiraly
      r1 = spiral.lastElement(); // zacatek spiraly
      spiral = getVect(r1, begin, 10); // najdi spiralu
      r2 = spiral.lastElement(); // konec spiraly
    } while (spiral.size() < 5); // "bodovy" artefakt

    r = new Row(img.getWidth() / 2, img.getHeight() / 2); // stred obrazku
    if (r1.distance2(r) > r2.distance2(r)) { // pokud spirala zacina na okraji
      spiral = spiral.invert();
    }
    engine.gui.showStatus("Potn ukoneno. Size: " + spiral.size());
    System.out.println("");
  }

  // Najde nejaky bod na spirale
  private Row getStart(Row r) {
    int x = r.getX() + 1, y = r.getY() + 1;
    while ((x < (img.getWidth() - 2)) && (y < (img.getHeight() - 1))) {
      x++; y++;
      if(isBlack(img.getRGB(x, y))) {
        return new Row(x,y);
      }
      if(isBlack(img.getRGB(x + 1, y))) {
        return new Row(x + 1, y);
      }
    }
    return begin; // vylezli jsme z obrazku
  }

  // vrati jeden plynuly usek spiraly bez vetvicek
  private SpiralVector getVect(Row now, Row last, int deep) {
    SpiralVector result = new SpiralVector(100);
    Vector vector = new Vector(4);
    result.add(now);
    Row r1, r2, p;
    boolean done = false;

    while (!done) {
      vector = getNeighbours(now, last); // najde vhodne sousedy
      done = (vector.size() == 0);
      if (!done) {
        last = now;
        r2 = r1 = (Row) vector.get(0);
        for (int j = 1; j < vector.size(); j++) {
          p = (Row) vector.get(j);
          if (p.distance2(now) < r1.distance2(now)) {
            r1 = p;
          }
          else if (p.distance2(now) < r2.distance2(now)) {
            r2 = p;
          }
          else {
            if (r1.distance2(r2) == 0) {
              r1 = p; // napriklad
            }
            else if (p.distance2(now) == r1.distance2(now)) {
              if ( ( (p.disX(r2) > 1) || (p.disY(r2) > 1)) &&
                  (r1.distance2(r2) == 1)) {
                r2 = p;
              }
            }
            else if (p.distance2(now) == r2.distance2(now)) {
              if ( ( (p.disX(r1) > 1) || (p.disY(r1) > 1)) &&
                  (r1.distance2(r2) == 1)) {
                r1 = p;
              }
            }
            else if ( (r1.distance2(now) == r2.distance2(now)) &&
                     (r2.distance2(now) == p.distance2(now))) {
              System.out.println("chyba - tri stejne vzdalene body");
            }
          }
        }
        if ( ( (r1.distance2(now) == 1) && (r2.distance2(now) == 1) &&
              (r1.distance2(r2) != 0)) || (r1.disX(r2) > 1) ||
            (r1.disY(r2) > 1)) { // rozdeleni typu T nebo V
          if (deep == 0) {
            System.out.println("Moc velk zanoen");
            return result;
          }
//          System.out.println("deleni: " + now + ", deep: " + deep);
          SpiralVector v1 = getVect(r1, now, deep - 1);
          SpiralVector v2 = getVect(r2, now, deep - 1);
          if (v2.size() < v1.size()) {
            SpiralVector pom = v1;
            v1 = v2;
            v2 = pom;
          }
          result.add(v1);
          result.add(v1.invert());
          result.add(now);
          result.add(v2);
          return result;
        }
        else if (r1.distance2(now) < r2.distance2(now)) {
          result.add(r1);
          now = r1;
        }
        else {
          result.add(r2);
          now = r2;
        }
      }
    }
    return result;
  }

  // najde vhodne sousedy k danemu bodu (now)
  private Vector getNeighbours(Row now, Row last) {
    Vector res = new Vector(4);
    for (int a = -1; a < 2; a++) {
      for (int b = -1; b < 2; b++) {
        Row p = new Row(now.getX() + a, now.getY() + b);
        if (p.distance2(now) != 0) { // neni to aktualni bod
          if (p.distance2(last) != 0) { // neni to minuly bod
            if (( (p.disX(last) == 0) && (p.disY(last) < 2)) || ( (p.disY(last) == 0) && (p.disX(last) < 2))) { // tyhle body jsme uz navstivili
              continue;
            }
            try {
              if (isBlack(img.getRGB(p.getX(), p.getY()))) {
                res.add(p);
              }
            }
            catch (ArrayIndexOutOfBoundsException e) {
              System.out.println("Mimo pole");
              return res;
            }
          }
        }
      }
    }
    return res;
  }

  /**
   * Ulozi vypocitany vektor bod do souboru
   * @param fileName jmno souboru
   */
  public void save(String fileName) {
    String str;
    try {
      BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new
          FileOutputStream(fileName)));
      Iterator i = spiral.iterator();
      int time = 0;
      while (i.hasNext()) {
        str = ( (Row) i.next()).toString() + " " + time + " 0";
        out.write(str, 0, str.length());
        out.newLine();
        time += 5;
      }
      out.close();
    }
    catch (IOException ex) {
      engine.gui.showStatus("Nelze vytvorit soubor");
    }
  }

  // je barva cerna?
  private boolean isBlack(int rgb) {
    return rgb == 0xff000000;
  }
}

class SpiralVector {
  Vector vec = null;

  public SpiralVector(int size) {
    vec = new Vector(size);
  }

  public void add(Row r) {
    vec.add(r);
  }

  public void add(SpiralVector v) {
    vec.addAll(v.vec);
  }

  public int size() {
    return vec.size();
  }

  public Iterator iterator() {
    return vec.iterator();
  }

  public Row lastElement() {
    return (Row) vec.lastElement();
  }

  public SpiralVector invert() {
    SpiralVector res = new SpiralVector(vec.size());
    for(int i = vec.size() - 1; i >= 0; i--) {
      res.add((Row) vec.get(i));
    }
    return res;
  }
}

class Row {
  private int x, y;

  public Row(int x, int y) {
    this.x = x;
    this.y = y;
  }

  public String toString() {
    return x + " " + y;
  }

  public int getX() {
    return x;
  }

  public int getY() {
    return y;
  }

  // vraci 2. mocninu vzdalenosti
  public int distance2(Row r) {
    return (((r.x - x)*(r.x - x)) + ((r.y - y)*(r.y - y)));
  }

  // vraci rozdil x-ovych slozek
  public int disX(Row r) {
    return Math.abs(r.x - x);
  }

  // vraci rozdil y-ovych slozek
  public int disY(Row r) {
    return Math.abs(r.y - y);
  }
}
