package spiral.test;

import spiral.all.*;
import java.awt.Graphics;
import java.io.File;

/**
 * <p>Title: Spiral analyzis</p>
 * <p>Description: Tda ovldajc ve :)</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Dolezel
 * @version 1.0
 */

public class Engine {
  private Data data = null;
  private Results res = null;
  private File file = null;

  /**
   * Tda, pes kterou se komunikuje s uivatelem
   */
  public GUI gui = null;

  /**
   * Vytvo nov Engine
   * @param f rozhran pro komunikaci
   */
  public Engine(GUI f) {
    this.gui = f;
  }

  /**
   * Nahraje spirlu ze souboru
   * @param fileName jmno souboru
   */
  public void loadFile(String fileName) {
    file = new File((new File(fileName).getName()));
    data = new Data(fileName, this);
    res = null;
  }

  /**
   * Ulo spirlu do souboru
   * @param fileName jmno souboru
   */
  public void saveSpiral(String fileName) {
    data.save(fileName);
  }

  /**
   * Ulo vpotan hodnoty do aktulnho adrese, do souboru output.txt
   */
  public void saveData() {
    if (res != null) {
      while(file.exists()) {
        file = new File(file.getName() + "!");
      }
      res.save(file.getName());
    }
    else {
      gui.mShow("Neni co ulozit");
    }
  }

  /**
   * Zpsob pekreslen spirly v GUI
   * @param g kde spirlu kreslit
   */
  public void repaint(Graphics g) {
    if (data != null) {
      data.paint(g);
    }
  }

  /**
   * Zalo nov vektor dat
   */
  public void newData(Configuration conf) {
    data = new Data(this, conf);
    res = null;
  }

  /**
   * Pid ke spirle dal bod
   * @param x x-ov souadnice
   * @param y y-ov souadnice
   * @param time as
   * @param pres tlak
   */
  public void newRow(int x, int y, int time, int pres) {
    data.newRow(x, y, time, pres);
  }

  /**
   * Spust vpoet pro spirlu
   */
  public void compute() {
    if ( (data != null) && (data.size() > 1)) {
      try {
        res = new Results(this, data);
      }
      catch (Exception e) {
        gui.mShow("Chyba vypoctu");
        e.printStackTrace();
      }
    }
    else {
      gui.mShow("Nemam data");
    }
  }

  /**
   * Metoda vzgeneruje spirlu dle zadanch parametr
   */
  public void generuj(Configuration conf) {
    int a = conf.getA();
    int b = conf.getB();
    int ampl1 = conf.getAM1();
    int ampl2 = conf.getAM2();
    double pomer = conf.getPOMER();
    double dfi2 = conf.getDFI();
    double ds = conf.getDS();
    double cfi = conf.getFI();
    int rx, ry, time = 0;

    double x1, y1, cosx, siny, zkos, r = 0, fi1 = 0, fi2 = 0, dfi1 = 0;
    this.newData(conf);
    boolean done = false;

    while (!done) {
      x1 = a * (fi1) * Math.cos(fi1);
      y1 = b * (fi1) * Math.sin(fi1);
      cosx = ampl1 * Math.sin(fi2) * Math.cos(fi1);
      siny = ampl1 * Math.sin(fi2) * Math.sin(fi1);
      zkos = ampl2 * Math.sin(fi1 / pomer);

      rx = (int) (x1 + cosx + zkos);
      ry = (int) (y1 + siny + zkos);
      r = Math.sqrt(rx * rx + ry * ry);
      this.newRow(rx, ry, time, 0);

//      System.out.println("fi: " + fi1 + " x: " + x1 + " y: " + y1 + " dfi: " + dfi2 + " fi: " + fi2 + " cosx: " + cosx + " siny: " + siny + " zkos: " + zkos);

      dfi1 = ds / 2 / (a + b) / Math.sqrt(1 + fi1 * fi1);
      fi1 += dfi1;
      fi2 += dfi2;
      time += 5;
      if (fi1 > cfi) {
        done = true;
      }
    }
    gui.grafRepaint();
  }
}
